import React, {
  useState,
  useImperativeHandle,
  useCallback,
} from "react";

import style from "./index.module.css";
import MyDialog from "@/components/mui/MyDialog";
import folderIcon from "@/assets/project/folderIcon.svg";
import bigFolderIcon from "@/assets/project/bigFolderIcon.svg";
import { useMessage } from "@/components/MySnackbar";
import MyTreeView from "@/components/mui/MyTreeView";
import CloudEController from "@/api/fileserver/CloudEController";

const MoveFile = (props: any) => {
  const { path, projectId, fileToken, currentOperateFile, selectIds, refresh } = props;
  const Message = useMessage();
  const [newPath, setNewPath] = useState("");

  const [treeData, setTreeData] = useState<any>([]);

  const getTree = useCallback(() => {
    if (fileToken && projectId) {
      return CloudEController.JobOutFileDirtree(
        "/",
        fileToken,
        projectId,
        false
      )?.then((res) => {
        if (Array.isArray(res.data)) {
          setTreeData(res.data);
        } else {
          setTreeData([]);
        }
      });
    }
  }, [fileToken, projectId]);

  let moveFileDialogRef: any = React.createRef();

  const showDialog = () => {
    moveFileDialogRef.current.handleClickOpen();
    getTree();
  };

  useImperativeHandle(props.onRef, () => {
    return {
      showDialog: showDialog,
    };
  });

  const handleMoveFileSubmit = () => {
    if (newPath) {
      if (!currentOperateFile) {
        // 批量移动
        const oldPaths = selectIds.map((name: any) => {
          return `${path}${path === "/" ? "" : "/"}${name}`;
        });
        CloudEController.JobFileBatchMove(
          newPath,
          oldPaths,
          "",
          fileToken,
          projectId
        )?.then((res) => {
          Message.success('移动成功！')
          refresh()
        });
      } else {
        const oldPath = `${path}${path === "/" ? "" : "/"}${
          currentOperateFile.name
        }`;
        if (oldPath === newPath) {
          Message.info("当前目录和目标目录一致，请重新选择目标目录");
          return;
        }
        CloudEController.JobFileMove(
          newPath,
          oldPath,
          "",
          fileToken,
          projectId
        )?.then((res) => {
          console.log(res);
        });
      }
    } else {
      Message.error("请选择移动到哪个目录");
    }
  };

  const renderLabel = (labelNmae: string) => {
    return (
      <span className={style.treeLabel}>
        <img className={style.labelFolderIcon} src={folderIcon} alt="" />
        {labelNmae}
      </span>
    );
  };

  const onNodeSelect = (a: any, b: any) => {
    console.log("onNodeSelect", a, b);
    setNewPath(b);
  };

  const moveFileSubmitloading = false;

  const idFunc = (item: any) => {
    return `${item.dir.substr(1)}${item.name}`;
  };

  return (
    <MyDialog
      handleSubmit={handleMoveFileSubmit}
      onRef={moveFileDialogRef}
      title="移动至"
      submitloading={moveFileSubmitloading}
    >
      <div className={style.rootTitle}>
        <img className={style.bigFolderIcon} src={bigFolderIcon} alt="" />
        ProjectData
      </div>
      <MyTreeView
        treeData={treeData}
        renderLabel={renderLabel}
        onNodeSelect={onNodeSelect}
        idFunc={idFunc}
        treeViewSx={{
          width: 400,
          overflow: "hidden",
        }}
      ></MyTreeView>
    </MyDialog>
  );
};

export default MoveFile;
