import React, { useState, useCallback, useEffect, useMemo } from "react";

import style from "./index.module.css";
import classnames from "classnames";
import { Button, InputBase, IconButton, TextField } from "@mui/material";
import LoadingButton from "@mui/lab/LoadingButton";
import { createTheme, ThemeProvider } from "@mui/material/styles";
import SearchIcon from "@mui/icons-material/Search";
import RefreshIcon from "@mui/icons-material/Refresh";
import Table from "@/components/Material.Ui/Table";
import bigFolderIcon from "@/assets/project/bigFolderIcon.svg";
import dataSetIcon from "@/assets/project/dataSetIcon.svg";
import fileIcon from "@/assets/project/fileIcon.svg";
import folderIcon from "@/assets/project/folderIcon.svg";
import noFile from "@/assets/project/noFile.svg";
import MyDialog from "@/components/mui/MyDialog";
import { useMessage } from "@/components/MySnackbar";
import { getDataFileToken } from "@/api/project_api";
import AddFolder from "./AddFolder";
import MoveFile from "./MoveFile";
import UpLoaderFile from "./UpLoaderFile";
import useMyRequest from "@/hooks/useMyRequest";
import { observer } from "mobx-react-lite";
import { useStores } from "@/store";
import CloudEController from "@/api/fileserver/CloudEController";
import { toJS } from "mobx";
import moment from "moment";
import Dialog from "@mui/material/Dialog";
import DialogActions from "@mui/material/DialogActions";
import DialogContent from "@mui/material/DialogContent";
import DialogContentText from "@mui/material/DialogContentText";
import DialogTitle from "@mui/material/DialogTitle";
import usePass from "@/hooks/usePass";

const theme = createTheme({
  palette: {
    neutral: {
      main: "#1370FF",
      contrastText: "#fff",
    },
  },
});
declare module "@mui/material/styles" {
  interface Palette {
    neutral: Palette["primary"];
  }

  interface PaletteOptions {
    neutral?: PaletteOptions["primary"];
  }
}

declare module "@mui/material/Button" {
  interface ButtonPropsColorOverrides {
    neutral: true;
  }
}

const ProjectData = observer(() => {
  const isPass = usePass();
  const Message = useMessage()
  const { currentProjectStore } = useStores();
  const [tableLoadding, setTableLoadding] = useState(false);
  const [deleteDialogOpen, setDeleteDialogOpen] = useState(false);
  const [currentOperateFile, setCurrentOperateFile] = useState<any>(null);
  const [deleteloading, setDeleteloading] = useState(false);
  const [path, setPath] = useState("/");
  const showPath = useMemo(() => {
    if (path === "/") {
      // return "ProjectData";
      return <span className={style.showPathSpan}>ProjectData</span>
    } else {
      console.log(path)
      console.log(path.split('/'))
      const pathArr = path.split('/')
      if (pathArr.length <= 4) {
        return (pathArr.map((item, index) => {
          return (<span onClick={()=>setPath(pathArr.slice(0, index + 1).join('/') === '' ? '' : pathArr.slice(0, index + 1).join('/'))} className={classnames({
            [style.showPathSpan]: true,
            [style.showPathSpanActive]: index === pathArr.length - 1
          })}>{index === 0 ? 'ProjectData' :item} {index === pathArr.length - 1 ? null : <i className={style.showPathI}>{'>'}</i>}</span>)
        }))
      } else {
        return (pathArr.map((item, index) => {
          return (<span onClick={()=>setPath(pathArr.slice(0, index + 1).join('/') === '' ? '' : pathArr.slice(0, index + 1).join('/'))} className={classnames({
            [style.showPathSpan]: true,
            [style.showPathSpanActive]: index === pathArr.length - 1
          })}>
            {index === 0 ? 'ProjectData' : index > pathArr.length - 4 ? item : '' }
            {/* && index > pathArr.length - 4 */}
            {index === pathArr.length - 1 || (index <= pathArr.length - 4 && index > 0) ? null : <i className={style.showPathI}>{'>'}</i>}
            {index === 1 && '...'}{index === 1 && <i className={style.showPathI}>{'>'}</i>}
          </span>)
        }))
      }
    }
  }, [path]);
  const [activeTab, setActiveTab] = useState(1);
  const [keyWord, setKeyWord] = useState("");
  const [fileToken, setFileToken] = useState("");

  const [list, setList] = useState<any>([]);

  const { run: getDataFileTokenRun } = useMyRequest(getDataFileToken, {
    onSuccess: (res: any) => {
      console.log("res", res);
      setFileToken(res.data);
    },
  });

  const projectId = useMemo(() => {
    return toJS(currentProjectStore.currentProjectInfo.id);
  }, [currentProjectStore]);

  const getList = useCallback(() => {
    if (fileToken && projectId) {
      setTableLoadding(true);
      return CloudEController.JobOutFileList(
        path,
        fileToken,
        projectId,
        false
      )?.then((res) => {
        setTableLoadding(false);
        if (Array.isArray(res.data)) {
          setList(res.data);
        } else {
          setList([]);
        }
      });
    }
  }, [fileToken, path, projectId]);
  useEffect(() => {
    getList();
  }, [getList]);
  
  // 全局搜索文件列表
  const searchFileList =  useCallback(() => {
    if (fileToken && projectId) {
      setTableLoadding(true);
        return CloudEController.JobSearchFileList(
          keyWord,
          path,
          fileToken,
          projectId,
          false
        )?.then((res) => {
          setTableLoadding(false);
          if (Array.isArray(res.data)) {
            setList(res.data);
          } else {
            setList([]);
          }
        });
    }
  }, [fileToken, path, projectId, keyWord]);

  const handleRefresh = () => {
    if (keyWord) {
      searchFileList()
    } else {
      getList();
    }
    setSelectIds([])
  };

  useEffect(() => {
    getDataFileTokenRun({
      id: currentProjectStore.currentProjectInfo.id as string,
    });
  }, [getDataFileTokenRun, currentProjectStore]);

  const handleKeyWordChange = (e: any) => {
    if (e.target.value.length > 30) {
      return
    }
    setKeyWord(e.target.value);
  };

  const handleKeyWordChangeKeyUp = (e: any) => {
    if (e.keyCode === 13) {
      searchFileList()
      console.log('搜索')
    }
  }

  const [selectIds, setSelectIds] = useState([]);

  const versionsHeadCells = [
    { id: "checkbox" },
    { id: "name", numeric: false, label: "名称", width: "25%" },
    { id: "size", numeric: false, label: "大小", width: "25%" },
    { id: "mtime", numeric: false, label: "创建时间", width: "25%" },
    { id: "caozuo", numeric: false, label: "操作", width: "25%" },
  ];

  const hanldeCheckbox = (e: any) => {
    console.log(e);
    setSelectIds(e);
  };

  const handleViewFolders = (item: any) => {
    console.log("handleViewFolders");
    console.log(item);
    if (path === '/') {
      setPath(`/${item.name}`)
    } else {
      setPath(`${path}/${item.name}`);
    }
  };

  const hanleDeleteFile = (item: any) => {
    setCurrentOperateFile(item);
    setDeleteDialogOpen(true);
  };

  const renderName = (item: any) => {
    if (item.type === "directory") {
      return (
        <span
          className={style.folderIconBox}
          onClick={() => handleViewFolders(item)}
        >
          <img className={style.folderIcon} src={folderIcon} alt="" />
          {item.name}
        </span>
      );
    } else {
      return (
        <span className={style.folderIconBox}>
          <img className={style.folderIcon} src={fileIcon} alt="" />
          {item.name}
        </span>
      );
    }
  };

  const renderSize = (item: any) => {
    return <span>{item.size ? `${item.size}b` : "-"}</span>;
  };

  const renderMtime = (item: any) => {
    return (
      // <span>{moment(new Date(item.mtime)).format("YYYY-MM-DD hh:mm:ss")}</span>
      <span>{moment(item.mtime).format("YYYY-MM-DD hh:mm:ss")}</span>
    );
  };

  const renderButtons = (item: any) => {
    return (
      <span>
        <Button
          sx={{ position: "relative", left: "-18px" }}
          variant="text"
          size="small"
        >
          下载
        </Button>
        <Button
          sx={{ position: "relative", left: "-18px" }}
          variant="text"
          size="small"
          onClick={() => hanleShowMoveFileDialog(item)}
        >
          移动至
        </Button>
        <Button
          sx={{ position: "relative", left: "-18px" }}
          variant="text"
          size="small"
          color="error"
          onClick={() => hanleDeleteFile(item)}
        >
          删除
        </Button>
      </span>
    );
  };

  let UpLoaderFileRef: any = React.createRef();
  const hanleShowUpLoaderFileDialog = () => {
    UpLoaderFileRef.current.showDialog();
  };

  let addFolderRef: any = React.createRef();

  const hanleShowAddFolderDialog = () => {
    addFolderRef.current.showDialog();
  };

  let moveFileRef: any = React.createRef();

  const hanleShowMoveFileDialog = (item: any) => {
    console.log(item);
    setCurrentOperateFile(item);
    moveFileRef.current.showDialog();
  };

  const handleDeleteDialogClose = () => {
    setDeleteDialogOpen(false);
  };

  const handleDelete = () => {
    let deletePath = ''
    if (currentOperateFile) {
      deletePath =
        path === "/"
          ? `${path}${currentOperateFile.name}`
          : `${path}/${currentOperateFile.name}`;
    } else {
      deletePath = selectIds.map((name: any) => {
        return `${path}${path === "/" ? "" : "/"}${name}`;
      }).join(' ');
    }
    CloudEController.JobOutFileDel(
      deletePath,
      fileToken,
      projectId as string
    )?.then((res) => {
      Message.success('删除成功')
      setDeleteloading(false);
      setDeleteDialogOpen(false);
      handleRefresh();
    });
  };

  const handleBatchMove = () => {
    setCurrentOperateFile(null);
    moveFileRef.current.showDialog();
  };

  const handleBatchDelete = () => {
    setCurrentOperateFile(null);
    setDeleteDialogOpen(true)
  }

  return (
    <ThemeProvider theme={theme}>
      <div className={style.projectData}>
        <div className={style.projectDataStickyTop}>
          <div className={style.projectDataTitle}>项目数据</div>
          <div className={style.projectDataHeader}>
            <div className={style.projectDataButtonAndSearch}>
              <div className={style.projectDataButtonBox}>
                <Button
                  color="neutral"
                  variant="contained"
                  size="small"
                  style={{ marginRight: "12px" }}
                  onClick={hanleShowUpLoaderFileDialog}
                  disabled={selectIds.length !== 0 || !isPass("PROJECT_DATA_UPLOAD")}
                >
                  {/* todo 项目权限 */}
                  上传文件
                </Button>
                <Button
                  color="neutral"
                  variant="outlined"
                  size="small"
                  onClick={hanleShowAddFolderDialog}
                  disabled={selectIds.length !== 0 || !isPass("PROJECT_DATA_ADDDIR")}
                  >
                  {/* todo 项目权限 */}
                  新建文件夹
                </Button>
              </div>
              <div className={style.projectDataSearch}>
                <InputBase
                  className={style.searchInput}
                  placeholder="输入关键词搜索"
                  inputProps={{ "aria-label": "输入关键词搜索" }}
                  value={keyWord}
                  onChange={handleKeyWordChange}
                  style={{ width: "280px", fontSize: "14px" }}
                  onKeyUp={handleKeyWordChangeKeyUp}
                />
                <IconButton
                  type="submit"
                  className={style.searchButton}
                  aria-label="search"
                  size="small"
                  style={{ padding: "4px" }}
                >
                  <SearchIcon
                    className={style.searchIcon}
                    style={{ color: "#999" }}
                  />
                </IconButton>
              </div>
            </div>
            <div className={style.projectDataPathAndTabs}>
              <div className={style.projectDataPath}>{showPath}</div>
              <div className={style.projectDataTabsAndBtton}>
                <div className={style.projectDataTabs}>
                  <div
                    className={classnames({
                      [style.projectDataTab]: true,
                      [style.projectDataTabActive]: activeTab === 1,
                    })}
                    onClick={() => setActiveTab(1)}
                  >
                    文件
                  </div>
                  <div
                    className={classnames({
                      [style.projectDataTab]: true,
                      [style.projectDataTabActive]: activeTab !== 1,
                    })}
                    onClick={() => setActiveTab(2)}
                  >
                    数据集
                  </div>
                </div>
                <IconButton
                  aria-label="refreshIcon"
                  size="small"
                  onClick={handleRefresh}
                >
                  <RefreshIcon />
                </IconButton>
              </div>
            </div>
          </div>
          <Table
            checkboxData={(e: any) => {
              hanldeCheckbox(e);
            }}
            // param="name"
            rowHover={true}
            stickyheader={true}
            rows={list.map((item: any) => ({
              ...item,
              id: item.name,
              name: renderName(item),
              size: renderSize(item),
              mtime: renderMtime(item),
              caozuo: renderButtons(item),
            }))}
            load={tableLoadding}
            // rowsPerPage={"99"}
            headCells={versionsHeadCells}
            nopadding={true}
            footer={false}
          ></Table>
        </div>
        {selectIds.length > 1 && (
          <div className={style.projectDataStickyBox}>
            <Button
              color="error"
              variant="outlined"
              size="small"
              style={{ marginRight: "12px" }}
              onClick={handleBatchDelete}
            >
              批量删除（{selectIds.length}）
            </Button>
            <Button
              color="neutral"
              variant="contained"
              size="small"
              style={{ marginRight: "24px" }}
              onClick={handleBatchMove}
            >
              批量移动（{selectIds.length}）
            </Button>
          </div>
        )}
        <Dialog
          open={deleteDialogOpen}
          onClose={handleDeleteDialogClose}
          aria-labelledby="提示"
          aria-describedby="确认要删除吗"
        >
          <DialogTitle id="alert-dialog-title">{"提示"}</DialogTitle>
          <DialogContent>
            <DialogContentText id="alert-dialog-description">
              确认要删除“{currentOperateFile ? currentOperateFile.name : selectIds.join('”，“')}
              ”吗？
            </DialogContentText>
          </DialogContent>
          <DialogActions>
            <Button
              onClick={handleDeleteDialogClose}
              color="inherit"
              variant="contained"
              style={{ color: "#1E2633", backgroundColor: "#fff" }}
              size="small"
            >
              取消
            </Button>
            <LoadingButton
              loading={deleteloading}
              onClick={handleDelete}
              color="primary"
              variant="contained"
              size="small"
            >
              确认
            </LoadingButton>
          </DialogActions>
        </Dialog>
        <UpLoaderFile
          onRef={UpLoaderFileRef}
          path={path}
          list={list}
        ></UpLoaderFile>
        <AddFolder
          onRef={addFolderRef}
          list={list}
          path={path}
          refresh={handleRefresh}
          fileToken={fileToken}
          projectId={projectId}
        ></AddFolder>
        <MoveFile
          onRef={moveFileRef}
          path={path}
          fileToken={fileToken}
          projectId={projectId}
          currentOperateFile={currentOperateFile}
          selectIds={selectIds}
          refresh={handleRefresh}
        ></MoveFile>
      </div>
    </ThemeProvider>
  );
});

export default ProjectData;
